# -*- coding: utf-8 -*-

import bpy

import os
from os import path
from mathutils import Vector
from pathlib import Path

class ModifierModule():

    def load_NodeGroup(self, node_name):

        if node_name in bpy.data.node_groups:
            return bpy.data.node_groups[node_name]

        dirname = path.dirname(__file__)
        addon_name = path.split(dirname)[-1]
        libfile = dirname + os.sep + addon_name + ".blend"

        with bpy.data.libraries.load(libfile) as (data_from, data_to):
            data_to.node_groups = [node_name]
            
        return bpy.data.node_groups[node_name]

    def add_GN_modifier(self, object, node_name, modifier_name = "GeometryNode"):

        # 命名規則が頭に GN. 付きの場合 
        if (modifier_name == "GeometryNode"):
            if ("GN." in node_name):
                modifier_name = node_name[3:]
                
        mod = object.modifiers.new(name = modifier_name, type="NODES")
        old_node = mod.node_group
        if (old_node != None):
            bpy.data.node_groups.remove(mod.node_group)
        node = self.load_NodeGroup(node_name)
        mod.node_group = node

        return mod 

    def make_node_dict(self, geometry_node):

        NodeDict = {}
        node_group = geometry_node.node_group

        if bpy.app.version >= (4,0,0):
            for item in node_group.interface.items_tree:
                if item.in_out == 'INPUT':
                    NodeDict[item.name] = item.identifier
        else:
            for input in node_group.inputs:
                NodeDict[input.name] = input.identifier
                
        return NodeDict

    def set_GN_inputs(self, mod, paramList):

        NodeDict = self.make_node_dict(mod)
        node_group = mod.node_group

        socket_name = {
            "object"    :("NodeSocketObject", "OBJECT"),
            "color"     :("NodeSocketColor" , "COLOR"),
            "vector"    :("NodeSocketVetor" , "VECTOR"),
            "value"     :("NodeSocketFloat" , "VALUE"),
            "int"       :("NodeSocketInt"   , "INT"),
            "bool"      :("NodeSocketBool"  , "BOOLEAN"),
            "collection":("NodeSocketCollection", "COLLECTION"),
        }

        for item in paramList:
            socket_key = NodeDict[item[0]]

            if bpy.app.version >= (4,0,0):
                node_input = node_group.interface.items_tree[item[0]]
                socket_type = node_input.socket_type
                version_index = 0
            else:
                node_input = node_group.inputs[item[0]]
                socket_type = node_input.type
                version_index = 1

            socket_object = socket_name["object"][version_index]
            
            if (not socket_type in [socket_object] and type(item[1]) == str):
                mod[socket_key+"_use_attribute"] = 1
                mod[socket_key+"_attribute_name"] = item[1]
            else:
                #型が違うとエラーになるので一部安全装置を
                if socket_type == socket_name["vector"][version_index]:
                    mod[socket_key][0] = float(item[1][0])
                    mod[socket_key][1] = float(item[1][1])
                    mod[socket_key][2] = float(item[1][2])
                elif socket_type == socket_name["value"][version_index]:
                    mod[socket_key] = float(item[1])
                    node_input.min_value = node_input.min_value
                    node_input.max_value = node_input.max_value
                elif socket_type == socket_name["int"][version_index]:
                    mod[socket_key] = int(item[1])
                elif socket_type == socket_name["bool"][version_index]:
                    mod[socket_key] = bool(item[1])
                elif socket_type == socket_name["object"][version_index]:
                    if (type(item[1]) == str):
                        if item[1] in bpy.data.objects:
                            mod[socket_key] = bpy.data.objects[item[1]]
                    else:
                        mod[socket_key] = item[1]
                elif socket_type == socket_name["collection"][version_index]:
                    if (type(item[1]) == str):
                        if (item[1] in bpy.data.collections):
                            mod[socket_key] = bpy.data.collections[item[1]]
                    else:
                        mod[socket_key] = item[1]
                else:
                    mod[socket_key] = item[1]

# make_node_dict:
# モディファイア上で見るソケットへの入力は["Input_4"]とかってなってるので
# 名前で設定できないので、対応する表を作る

# set_GN_inputs:
# make_node_dict で作ったリストを使って、名前でパラメータをセットできる
# paramList はこんな感じ
# paramList = [("Scale", self.scale"),
#              ("Rot", self.rot),
#              ("Col", "Col")]
# 文字列で指定した場合は、アトリビュート扱いする


#リストではなく直接入力も可能に
    def modifier_input(self, mod, label, value):

        socket_name = {
            "object"    :("NodeSocketObject", "OBJECT"),
            "color"     :("NodeSocketColor" , "COLOR"),
            "vector"    :("NodeSocketVector" , "VECTOR"),
            "value"     :("NodeSocketFloat" , "VALUE"),
            "int"       :("NodeSocketInt"   , "INT"),
            "bool"      :("NodeSocketBool"  , "BOOLEAN"),
            "collection":("NodeSocketCollection", "COLLECTION"),
        }

        if bpy.app.version >= (4,0,0):
            inputs = mod.node_group.interface.items_tree
            version_index = 0
        else:
            inputs = mod.node_group.inputs
            version_index = 1

        for input in inputs:
            if (bpy.app.version >= (4, 0, 0)):
                if input.in_out == "OUTPUT":
                    continue

            print(input.name, label)
            if input.name == label:
                if bpy.app.version >= (4,0,0):
                    socket_type = input.socket_type
                else:
                    socket_type = input.type

                if socket_type == socket_name["vector"][version_index]:
                    mod[input.identifier][0] = value[0]
                    mod[input.identifier][1] = value[1]
                    mod[input.identifier][2] = value[2]
                elif socket_type == socket_name["value"][version_index]:
                    mod[input.identifier] = float(value)
                elif socket_name == socket_name["int"][version_index]:
                    mod[input.identifier] = int(value)
    
